<?php

	class Pushengage
	{
		private static $pushengage = null;
		public static $pushengage_version = '2.0.2';
		public static $database_version = '29-01-2020';
		//Constructor Function
		public function __construct()
		{
			//blank
			global $wp_session;
		}

		//Init Function
		public static function init()
		{
			if ( is_null( self::$pushengage ) )
			{
				self::$pushengage = new self();
				$pushengage_settings = self::pushengage_settings();
				if ( empty( $pushengage_settings ) 
					|| self::$pushengage_version !== $pushengage_settings['version'] )
				{
					self::install( $pushengage_settings );
				}

				self::add_actions();
				
			}
			return self::$pushengage;
		}

		public function checkUserAuthenticaiton($app_key)
		{
			if( !empty($app_key) 
				&& ! empty($_GET['page']) 
				&& $_GET['page'] == 'pushengage-admin' 
				&& empty($wp_session['check_auth'])
			)
			{
				$wp_session['check_auth'] = Api_class::verifyUser($app_key);
				return $wp_session['check_auth'];
			}
			else
				return false;
		}

		public static function print_my_inline_script()
		{
			$script =  "window._peq = window._peq || []; window._peq.push(['init']);";
			wp_add_inline_script('core', $script, 'before');
		}

		//Actions Function
		public static function add_actions()
		{
			$pushengage_settings = self::pushengage_settings();
			
			if (!empty($pushengage_settings)) 
			{
				add_action( 'wp_enqueue_scripts', array( __CLASS__, 'pushengageDomainScripts') );
				add_action( 'transition_post_status', array( __CLASS__, 'sendPostNotifications' ), 10, 3 );
				
				//If subscription popup is not blocked then only we add async to core script
				if($pushengage_settings['disable_subscription_popup'] != true)
				{
					//Include subscritpion popup async function
					add_action( 'wp_enqueue_scripts', array( __CLASS__, 'print_my_inline_script') );
				}

				//Include subscripiton popup core script with async mode
				add_filter( 'script_loader_tag', function ( $tag, $handle ) {
					if ( 'core' !== $handle )
						return $tag;
				
					return str_replace( ' src', ' async src', $tag );
				}, 10, 2 );
			}
		
			if ( is_admin() )
			{
				add_action( 'init', array( __CLASS__, 'init_user_options' ) );
			}

		}

		public static function pushengage_settings()
		{
			return get_option( 'pushengage_settings' );
		}

		public static function pushengage_active()
		{
			$pushengage_settings = self::pushengage_settings();
			$app_key = $pushengage_settings['appKey'];
			if ( ! empty( $app_key ) ) {
				return true;
			} else {
				return false;
			}
		}

		public static function note_override_add_meta_box() {
			$pushengage_settings = self::pushengage_settings();
			if(!empty($pushengage_settings['all_post_types']))
			$screens = get_post_types();
			else
			$screens = array('post');	
			foreach ( $screens as $screen ) 
			{
				add_meta_box(
					'pushengage_notif_on_post',
					'PushEngage Push Notification',
					array( __CLASS__, 'note_override' ),
					$screen,
					'side',
					'high'
				);
			}	
		}



		public static function note_override()
		{
			if(empty($wp_session['pushengage_settings']))
			{
				$pushengage_settings = self::pushengage_settings();
				$wp_session['pushengage_settings'] = $pushengage_settings;
			}			
			else
			$pushengage_settings = $wp_session['pushengage_settings'];

			$app_key = $pushengage_settings['appKey'];

			$auto_push = $pushengage_settings['autoPush'];
			$all_post_types = $pushengage_settings['all_post_types'];
			global $post;
			$check_auth = !empty($wp_session['check_auth'])?$wp_session['check_auth']:'';
			if(!empty($check_auth['block_user']))
				return false;

			if ( 'post' === $post->post_type || true === $all_post_types )
			{
				printf('<div style="padding-left:10px;padding-bottom:15px" id="pushengage-post-checkboxes">');
				$display_segments_div = " display:none; ";
				if ( 'auto-draft' === $post->post_status )
				{
					//We need to check send notification check box if auto_push is true otherwise we should uncheck this./
					if ( true === $auto_push ) {
						printf( '<label><input type="checkbox" value="1" checked id="pushengage-override-checkbox" name="pushengage-override" style="margin: -3px 9px 0 1px;" checked onclick="selectSubscriberSegments()"/>');
						echo 'Send PushEngage Notification</label>';
						$display_segments_div = "style='display:block'";
					} else {
						printf( '<label><input type="checkbox" value="1" id="pushengage-override-checkbox" name="pushengage-override" style="margin: -3px 9px 0 1px;" onclick="selectSubscriberSegments()"/>');
						echo 'Send PushEngage Notification</label>';
					}
				}
				else
				{
					//check for over ride and scheduled options.
					$pe_override = get_post_meta( $post->ID, '_pe_override', true );
					$pe_scheduled = get_post_meta( $post->ID, 'pe_override_scheduled', true );
					//By default we are unchecking published posts. That means when ever user edits send notification check box unchecked.
					if(($pe_override == '1' || $pe_scheduled == '1') && $post->post_status!='publish')
					{
						$upd_chk_box = 'checked';
						$display_segments_div = " display:block; ";
					}
					else
					$upd_chk_box = '';

					printf('<label><input type="checkbox" value="1" id="pushengage-override-checkbox" name="pushengage-override" style="margin: -3px 9px 0 1px;" '.$upd_chk_box.'  onclick="selectSubscriberSegments()"/>');
					echo 'Send PushEngage Notification on Update</label>';
				}
				//check all subscribers when click on send notification
				$draft_segments = get_post_meta( $post->ID, '_pe_draft_segments', true );
				if(!empty($draft_segments))
				{
					$draft_segments = explode(' ', $draft_segments);
				}
				echo "<script>
						function selectSubscriberSegments()
						{
							if(document.getElementById('pushengage-override-checkbox').checked == true)
							{
								var drft_seg ='".$draft_segments."';
								if(!drft_seg && document.getElementById('selectall'))
								document.getElementById('selectall').checked=true;
								if(document.getElementById('pushengage-post-categories'))
								document.getElementById('pushengage-post-categories').style.display = 'block';
							}
							else
							{
								if(document.getElementById('selectall'))
								document.getElementById('selectall').checked=false;
								if(document.getElementById('pushengage-post-categories'))
								document.getElementById('pushengage-post-categories').style.display = 'none';
							}
						}
					  </script>
					 ";
				wp_nonce_field( 'pushengage_save_post', 'hidden_pe' );
				echo '</div>';

					if(empty($wp_session['segmets_data']))
					$segmets_data = Api_class::getSegments($app_key);
					else
					$segmets_data = $wp_session['segmets_data'];

					if(!empty($segmets_data["segments"]))
					{
						$pe_override = get_post_meta( $post->ID, '_pe_override', true );

						if($auto_push || $all_post_types || empty($draft_segments))
							$check ="checked";
						else
							$check ="";

						if($post->post_status == 'auto-draft' && ($auto_push || $all_post_types))
						$check ='checked';

						printf('<div style="padding-left:37px;padding-top:0px;padding-bottom:10px;'.$display_segments_div.'" id="pushengage-post-categories"><span style="font-weight:bold;">Select PushEngage Segments</span>');
						echo '<br><input type="checkbox" id="selectall" '. $check.' onclick="selectAll();"><span  style="margin-left:10px;">All Subscribers</span>';
						foreach($segmets_data["segments"] as $segment)
						{
							if(!empty($draft_segments) && in_array($segment["segment_id"], $draft_segments))
							$seg_chk_box = ' checked ';
							else
							$seg_chk_box = '';

							if(!empty($segment["segment_name"]))
							{
								echo '<div style="margin:5px 10px 5px 0px !important;"><input type="checkbox"   '.$seg_chk_box.'class="pushengage-segments" onclick="selectAllSubscribers()" name="pushengage-categories[]" value="'.$segment["segment_id"].'" ><span style="margin-left:10px;">'.$segment["segment_name"].'</span></div>';
							}
						}
						echo '</div>';
						echo '<script>
						function selectAll()
						{
							var all_cb = document.getElementById("selectall").checked;
							var pe_segments = document.getElementsByClassName("pushengage-segments");

							for (var key in pe_segments)
							{
							  if (pe_segments.hasOwnProperty(key))
							  {
								if(all_cb)
								{
									pe_segments[key].checked = false;
								}
								else
								{
									pe_segments[key].checked = true;
								}
							  }
							}
						}

						function selectAllSubscribers()
						{
							var pe_segments = document.getElementsByClassName("pushengage-segments");
							var check_flag = false;
							for (var key in pe_segments)
							{
								if(pe_segments[key].checked == true)
								check_flag = true;
							}
							if(check_flag==false)
							{
								document.getElementById("selectall").checked = true;
							}
							else
							{
								document.getElementById("selectall").checked = false;
							}
						}

						</script>';
					}

			}
		}

		public static function custom_note_text( $post_type, $post )
		{
			$pushengage_settings = self::pushengage_settings();
			$all_post_types = $pushengage_settings['all_post_types'];
			if ( 'post' === $post_type || true === $all_post_types ) {
				if ( 'attachment' !== $post_type && 'comment' !== $post_type && 'dashboard' !== $post_type && 'link' !== $post_type ) {
					add_meta_box(
						'pushengage_meta',
						'Custom Notification Title',
						array( __CLASS__, 'pushengage_custom_headline_content' ),
						'',
						'normal',
						'high'
					);
				}
			}
		}

		public static function pushengage_custom_headline_content( $post )
		{
			$custom_note_text = get_post_meta( $post->ID, '_pushengage_custom_text', true );
			?>
			<div id="pushengage-custom-note" class="form-field form-required">
				<input type="text" id="pushengage-custom-note-text" maxlength="73" placeholder="Enter Custom Headline For Your Notification" name="pushengage-custom-msg" value="<?php echo ! empty( $custom_note_text ) ? esc_attr( $custom_note_text ) : ''; ?>" /><br>
				<span id="pushengage-custom-note-text-description" >Custom headline limit 73 characters.<br/>When using a custom headline, this text will be used in place of the default blog post title for your push notification.</span>
			</div>
		<?php
		}

		public static function pushengage_save_settings()
		{
			if ( isset( $_POST['action']) && $_POST['action'] == 'update_wordpress_settings' && $_GET['page'] == 'pushengage-admin')
			{

				$pushengage_settings = self::pushengage_settings();
				$app_key = $pushengage_settings['appKey'];
				$auto_push = $pushengage_settings['autoPush'];
				$bbPress = $pushengage_settings['bbPress'];
				$prompt_min = $pushengage_settings['prompt_min'];
				$prompt_visits = $pushengage_settings['prompt_visits'];
				$prompt_event = $pushengage_settings['prompt_event'];
				$non_pushengage_categories = $pushengage_settings['categories'];
				$segment_send = $pushengage_settings['segment_send'];
				$use_custom_script = false;
				$custom_script = '';
				$use_featured_image = $pushengage_settings['use_featured_image'];
				$all_post_types = $pushengage_settings['all_post_types'];
				$utmcheckbox = $pushengage_settings['utmcheckbox'];
				$utm_source = $pushengage_settings['utm_source'];
				$utm_medium = $pushengage_settings['utm_medium'];
				$utm_campaign = $pushengage_settings['utm_campaign'];
				$disable_subscription_popup = $pushengage_settings['disable_subscription_popup'];
				
				if (isset( $_POST['action_settings'] ) && $_POST['action_settings']=='post')
				{
			    	$auto_push = isset($_POST['pushengage-auto-push']) ? true : false;
			    	$all_post_types = isset($_POST['pushengage-all-post-types']) ? true : false;
			    	$disable_subscription_popup = isset($_POST['disable_subscription_popup']) ? true: false;

					if(isset( $_POST['pushengage-custom-image'] ) && $_POST['pushengage-custom-image'] == 1)
					{
						$use_featured_image = true;
					}
					else
					{
						$use_featured_image = false;
					}
				}

				if (isset( $_POST['bbPress'])) {
					$bbPress = true;	
				}

				if ( isset( $_POST['pushengage-prompt-min'] ) ) {
						$prompt_min = true;
				}

				if ( isset( $_POST['pushengage-prompt-visits'] ) ) {
					if ( '0' === $_POST['pushengage-prompt-visits'] || '1' === $_POST['pushengage-prompt-visits'] ) {
						$prompt_visits = 2;
					} else {
						$prompt_visits = sanitize_text_field( $_POST['pushengage-prompt-visits'] );
					}
				}

				if ( isset( $_POST['pushengage-prompt-event'] ) ) {
					$prompt_event = true;
				}

				if ( isset( $_POST['pushengage-categories'] ) ) {
					$non_pushengage_categories = array_map( sanitize_text_field, $_POST['pushengage-categories'] );
				}
				
				if ( isset( $_POST['pushengage-segment-send'] ) ) {
					$segment_send = true;
				}

				if (isset( $_POST['action_settings'] ) && $_POST['action_settings']=='utm')
				{
				    if(isset( $_POST['utmcheckbox'] ))
                    {
                        $utmcheckbox = true;
                        if ( isset( $_POST['utm_source'] ) ) {
                            $utm_source = sanitize_text_field($_POST['utm_source']);
                        }
                        if ( isset( $_POST['utm_medium'] ) ) {
                            $utm_medium = sanitize_text_field($_POST['utm_medium']);
                        }
                        if ( isset( $_POST['utm_campaign'] ) ) {
                            $utm_campaign = sanitize_text_field($_POST['utm_campaign']);
                        }
                    }
				    else
                    {
                        $utmcheckbox = false;
                        $utm_source = '';
                        $utm_medium = '';
                        $utm_campaign = '';
                    }
				}
				

				$custom_script = !empty($_POST['pushengage-custom-script'])?sanitize_text_field( $_POST['pushengage-custom-script'] ):'';

				$form_data = array(
					'app_key' => $app_key,
					'autoPush' => $auto_push,
					'bbPress' => $bbPress,
					'prompt_min' => $prompt_min,
					'prompt_visits' => $prompt_visits,
					'prompt_event' => $prompt_event,
					'categories' => $non_pushengage_categories,
					'segment_send' => $segment_send,
					'use_custom_script' => $use_custom_script,
					'custom_script' => $custom_script,
					'use_featured_image' => $use_featured_image,
					'utmcheckbox' => $utmcheckbox,
					'utm_source' => $utm_source,
					'utm_medium' => $utm_medium,
					'utm_campaign' => $utm_campaign,
					'all_post_types' => $all_post_types,
					'disable_subscription_popup' => $disable_subscription_popup
				);

				//update site info
				$site_data = array();
				if(!empty($_POST['site_name']))
					$site_data['site_name'] = sanitize_text_field($_POST['site_name']);
				if(!empty($_POST['site_url']))
					$site_data['site_url'] = sanitize_text_field($_POST['site_url']);
				if(!empty($_POST['site_image']))
					$site_data['site_image'] = sanitize_text_field($_POST['site_image']);

				if(!empty($site_data)) {
					$result = Api_class::updateSiteSettings($app_key, $site_data);
					//get the updated site data
					$appdata = self::getSiteData($app_key);
					$appdata = $appdata[0];
					$wp_session['appdata'] = $appdata;
				}

				self::update_settings( $form_data );
				wp_redirect( esc_url_raw( admin_url( 'admin.php?page=pushengage-admin&tab=gSettings' ) . '&status=success'));
				exit;
			}
		}

		public static function install( $pushengage_settings )
		{
			if ( empty( $pushengage_settings ) )
			{
				$pushengage_settings = array(
					'appKey' => '',
					'appSecret' => '',
					'version' => self::$pushengage_version,
					'autoPush' => true,
					'bbPress' => true,
					'database_version' => self::$database_version,
					'prompt_min' => false,
					'prompt_visits' => 2,
					'prompt_event' => false,
					'categories' => array(),
					'segment_send' => false,
					'use_custom_script' => false,
					'custom_script' => '',
					'chrome_error_dismiss' => false,
					'gcm_token' => '',
					'use_featured_image' => true,
					'use_require_interaction' => true,
					'all_post_types' => true,
					'utmcheckbox' => false,
					'utm_source' => '',
					'utm_medium' => '',
					'utm_campaign' => '',
					'disable_subscription_popup' => false,
				);

				add_option( 'pushengage_settings', $pushengage_settings );
			}
			if ( !empty($pushengage_settings['version']) && self::$pushengage_version !== $pushengage_settings['version'] )
			{
				self::update( $pushengage_settings );
			}
		}

		public static function update( $pushengage_settings )
		{
			$pushengage_settings['version'] = self::$pushengage_version;
			if(empty($pushengage_settings['site_name']))
			{
				if(!empty($wp_session['appdata']['site_name']))
				$pushengage_settings['site_name'] = $wp_session['appdata']['site_name'];
				else
				{
					if(!empty($_POST['appid']))
					{
						$appdata = self::getSiteData(sanitize_text_field($_POST['appid']));
						$appdata = $appdata[0];

						if(isset($appdata['site_name']) && !empty($appdata['site_name']))
						$pushengage_settings['site_name'] = $appdata['site_name'];
					}
				}
			}
			update_option( 'pushengage_settings', $pushengage_settings );
		}

		public static function admin_menu_add()
		{
			add_menu_page(
				'Pushengage',
				'PushEngage',
				'manage_options',
				'pushengage-admin',
				array( __CLASS__, 'admin_menu_page' ),
				PUSHENGAGE_URL . 'images/pe_logo.png'
			);
		}

		public static function admin_menu_page()
		{

			$pushengage_settings = self::pushengage_settings();
			$app_key = $pushengage_settings['appKey'];
			$cat_args = array(
				'hide_empty' => 0,
				'order' => 'ASC'
			);

			if(empty($app_key))
			{
				$wp_session['menu_active_key'] = false;
				$status = "";
			}

			if(isset($_POST['appid']) && $_POST['appid'] && $_POST['action'] == 'update_app_key')
			{	
				$tab_start = "setup";
				$appdata = self::getSiteData(sanitize_text_field($_POST['appid']));
				$appdata = $appdata[0];
				$wp_session['appdata'] = $appdata;

				if(!empty($appdata))
				{
					$wp_session['menu_active_key'] = true;
					$pushengage_settings['appKey'] = sanitize_text_field($_POST['appid']);

					if($appdata['site_name'])
						$pushengage_settings['site_name'] = $appdata['site_name'];

					if($appdata['site_key'])
						$pushengage_settings['site_key'] = $appdata['site_key'];

					self::update($pushengage_settings);
					$pushengage_settings = self::pushengage_settings();
					$app_key = $pushengage_settings['appKey'];
				}
				else
				{
					$wp_session['menu_active_key'] = false;
					$status = "failed";
				}
			}

			if(!empty($app_key))
			{
					//check user authentication
					$wp_session['check_auth'] = self::checkUserAuthenticaiton($app_key);

					if(empty($wp_session['check_auth']['error_code']))
					{
						$wp_session['menu_active_key'] = true;
					}
					else 
					{
						$wp_session['menu_active_key'] = false;
					}
					//Site Info
					if(empty($wp_session['appdata']))
					{
						$appdata = self::getSiteData($app_key);
						if(!empty($appdata[0]))
						{
							$appdata = $appdata[0];
							$wp_session['appdata'] = $appdata;
						}
					}

				//Get general settings
				if(empty($_GET['tab']) || ($_GET['tab'] == 'gSettings') && $_GET['page']=='pushengage-admin')
				{
					// Get general settings
					if(empty($wp_session['tabdata']))
					{
						$general_settings = Api_class::getGeneralSettings($app_key);
						$wp_session['tabdata'] = $general_settings;
					}
				}

				//Get subscription popup settings
				if(!empty($_GET['tab']) && $_GET['tab'] == 'subDialogbox' && empty($wp_session['optin_settings_data']))
				{
					if(empty($wp_session['tabdata']))
					{
						
						$optin_settings_data = Api_class::getSubscriptionPoupSettings($app_key);
						$wp_session['tabdata'] = $optin_settings_data;
						if(!empty($optin_settings_data['site_info']['optin_settings']))
						{
							$optin_settings = json_decode($optin_settings_data['site_info']['optin_settings']);
							$wp_session['tabdata']['optin_settings_data'] = $optin_settings;
						}

						if(!empty($optin_settings_data['segments']))
						{
							$wp_session['tabdata']['segments_data'] = $optin_settings_data['segments'];
						}

						if(!empty($optin_settings_data['site_type']))
						{
							$wp_session['tabdata']['site_type'] = $optin_settings_data['site_type'];
						}
					}
				}

				if (!empty($_POST['action']) && $_POST['action'] == "update_optin_settings")
				{
					$tab = "subDialogbox";

					//XSS Fix
					foreach ($_POST as $key => $value) 
					{
						if( gettype($_POST[$key])=="array")
						{
						$_POST[$key]= array_map(function($v){
									return trim(strip_tags($v));
						}, $_POST[$key]);
						}else
						$_POST[$key] = strip_tags($_POST[$key]);
				 	}

					$optin_segments = !empty($_POST['segments']) ? json_encode(array_map(sanitize_text_field,$_POST['segments'])) : '{}';
					if(isset($_POST['plan-switch']) && $_POST['plan-switch'] == "on")
					{

						if(isset($_POST['optin_sw_support']) && $_POST['optin_sw_support'] == "on" && !in_array($_POST['optin_type'], array(4,5)))
						{
							$quick_install = false;
						}
						else
						{
							$quick_install = true;
						}

						$optindata = array(
							'desktop'=> array(
								'http' => $wp_session['tabdata']['optin_settings_data']->desktop->http,
								'https' => array( 
									'optin_delay' => intval(sanitize_text_field($_POST['optin_delay'])),
									'optin_type' => intval(sanitize_text_field($_POST['optin_type'])),
									'optin_title' => sanitize_text_field($_POST['optin_title']),
									'optin_allow_btn_txt' => sanitize_text_field($_POST['optin_allow_btn_txt']),
									'optin_close_btn_txt' => sanitize_text_field($_POST['optin_close_btn_txt']),
									'optin_font' => '',
									'optin_sw_support' => $quick_install,
									'optin_segments' => json_encode(array_map(sanitize_text_field,$_POST['segments']))
					 			)
							),
							'mobile' => '',
							'intermediate' => $wp_session['tabdata']['optin_settings_data']->intermediate
						);
					}
					else
					{
						$optindata = array(
							'desktop'=> array(
								'http' => array( 
									'optin_delay' => intval(sanitize_text_field($_POST['optin_delay'])),
									'optin_type' => intval(sanitize_text_field($_POST['optin_type'])),
									'optin_title' => sanitize_text_field($_POST['optin_title']),
									'optin_allow_btn_txt' => sanitize_text_field($_POST['optin_allow_btn_txt']),
									'optin_close_btn_txt' => sanitize_text_field($_POST['optin_close_btn_txt']),
									'optin_font' => '',
									'optin_segments' => json_encode(array_map(sanitize_text_field,$_POST['segments']))
					 			),
								'https' => $wp_session['tabdata']['optin_settings_data']->desktop->https
							),
							'mobile' => '',
							'intermediate'=> $wp_session['tabdata']['optin_settings_data']->intermediate
							);
					}

					if(isset($_POST['plan-switch']) && $_POST['plan-switch'] == "on")
					{
						$site_type = 'https';
					}
					else
					{
						$site_type = 'http';
					}

					$data = array(
						'option_data' => $optindata,
						'site_type' => $site_type
					);					

					//update optin settings
					$result = Api_class::updateSubscriptionboxSettings($app_key, $data);

					$wp_session['tabdata']['optin_settings'] = json_encode($optindata);
					$wp_session['tabdata']['optin_settings_data'] = json_decode(json_encode($optindata));
					$wp_session['tabdata']['site_type'] = $site_type;
				}

				if (!empty($_POST['action']) && $_POST['action']=="update_optin_page_settings")
				{
					$tab = "subDialogbox";

					//XSS Fix
					$_POST = array_map(function($v) {
						return trim(strip_tags($v));
					}, $_POST);

					$optindata = array(
						'desktop' => $wp_session['tabdata']['optin_settings_data']->desktop,
						'mobile'=> $wp_session['tabdata']['optin_settings_data']->mobile,
						'intermediate'=> array(
							'page_heading' => sanitize_text_field($_POST['page_heading']), 
							'page_tagline'=> sanitize_text_field($_POST['page_tagline'])
						)
					);

					$data = array(
						'site_id' => $wp_session['appdata']['site_id'],
						'type' => 'update_optin_settings',
						'option_data' => $optindata
					);
					
					$result = Api_class::updateOptinSettings($app_key, $data);
					$wp_session['tabdata']['optin_settings'] = json_encode($optindata);
					$wp_session['tabdata']['optin_settings_data'] = json_decode(json_encode($optindata));
				}

				// Update welcome notification settings
				if (!empty($_POST['action']) && $_POST['action']=="save_welcome_notification")
				{
					$tab = "welcome_notification";
					if(isset($_POST['welcome_enabled']))
					{	
						if($_POST['welcome_enabled']=="true")
						{
							$welcome_enabled = "true";
						}
						else
						{
							$welcome_enabled = "false";
						}
					}

					$title = sanitize_text_field($_POST['notification_title']);
					$message = sanitize_text_field($_POST['notification_message']);
					$url = sanitize_text_field($_POST['display_notification_url']);
					$data = array(
						'site_id' => $wp_session['appdata']['site_id'],
						'type' => 'update_welcome_notification',
						'notification_title' => $title,
						'notification_message' => $message,
						'notification_url'	=> $url,
						'welcome_enabled'	=> $welcome_enabled
					);
					$welcome_note_data = Api_class::updateWelcomeNotification($app_key, $data);
					$wc_data = array(
						'notification_title' => $title,
						'notification_message' => $message,
						'notification_url' => $url,
						'welcome_enabled' => $welcome_enabled
					);
				 }
				
				//Get welcome notification settings
				if(!empty($_GET['tab']) && $_GET['tab'] == 'welcome_notification' && empty($wp_session['welcome_note_data']))
				{
					if(empty($wp_session['tabdata']))
					{
						$welcome_note_data = Api_class::getWelcomeNotificationSettings($app_key);
						$wp_session['tabdata'] = $welcome_note_data;

						if(!empty($welcome_note_data['welcome_notification_info'])) 
						{

							$wp_session['tabdata']['welcome_note_data'] = json_decode($welcome_note_data['welcome_notification_info']['option_value']);
						}
					}
				}

				//Update general settings
				if (!empty($_POST['action']) && $_POST['action']=="update_site_settings")
				{
					$tab = "gSettings";
					$site_name = sanitize_text_field($_POST['site_name']);
					$site_url = sanitize_text_field($_POST['site_url']);
					$site_image = sanitize_text_field($_POST['site_image']);
					$max_notifications = intval(sanitize_text_field($_POST['max_notifications'])); 

					$data = array(
						'site_id' => $wp_session['appdata']['site_id'],
						'type' => 'update_site_settings',
						'site_name' => $site_name,
						'site_url' => $site_url,
						'site_image' => $site_image,
						'max_notifications' => $max_notifications
					);

					$result = Api_class::updateSiteSettings($app_key, $data);
					$wp_session['tabdata']['site_info']['site_name'] = $site_name;
					$wp_session['tabdata']['site_info']['site_url'] = $site_url;
					$wp_session['tabdata']['site_info']['site_image'] = $site_image;
					$wp_session['tabdata']['site_info']['max_notifications'] = $max_notifications;

					if(!empty($wp_session['tabdata']['site_info']['site_name']))
					{
						$pushengage_settings['site_name'] = $wp_session['tabdata']['site_info']['site_name'];

						self::update($pushengage_settings);
					}

				}

				if (!empty($_POST['action']) && $_POST['action']=="update_profile")
				{
					$tab = "insSettings";
					$data = array(
						'site_id' => $wp_session['appdata']['site_id'],
						'type'	  => 'update_profile_settings',
						'user_name'	  => sanitize_text_field($_POST['user_name']),
						'timezones'	  => sanitize_text_field($_POST['timezones'])
					);

					$result = Api_class::updateProfileSettings($app_key, $data);
					$wp_session['tabdata']['user_info']['user_name'] = sanitize_text_field($_POST['user_name']);
					$wp_session['tabdata']['timezone_info']['option_value'] = sanitize_text_field($_POST['timezones']);
				}

				//Get FCM settings
				if(!empty($_GET['tab']) && $_GET['tab'] == 'gcmSettings' && empty($wp_session['gcmdata']))
				{
					if(empty($wp_session['tab']))
					{
						//fcm settings
						$fcmdata = Api_class::getFCMSettings($app_key);
						$wp_session['tabdata'] = $fcmdata;
						if(!empty($fcmdata['gcm_info']))
						$wp_session['tabdata']['gcmdata'] = json_decode($fcmdata['gcm_info']['option_value']);
						else 
						{
							$fcmdata['gcm_api_key'] = !empty($_POST['gcm_api_key'])? sanitize_text_field($_POST['gcm_api_key']):'';
							$fcmdata['gcm_project_key'] = !empty($_POST['gcm_project_key'])? sanitize_text_field($_POST['gcm_project_key']):'';
							$wp_session['tabdata']['gcmdata'] = (object)$fcmdata;
						}
					}
				}

				if(isset($_POST['verify_gcm']))
				{
					$tab = "gcmSettings";
					$data = array('gcm_api_key'=>sanitize_text_field($_POST['gcm_api_key']),'type'=>'verify_gcm');
					$verify_gcm_result = Api_class::verifyFCM($app_key, $data);
				}
				else if (!empty($_POST['action']) && $_POST['action']=="update_gcm_settings")
				{
					$tab = "gcmSettings";
					$error_message = "";

					//verify GCM Key
					$verify_gcm_data = array(
						'gcm_api_key' => sanitize_text_field($_POST['gcm_api_key']),
						'type' => 'verify_gcm'
					);
					$verify_gcm_result = Api_class::verifyFCM($app_key, $verify_gcm_data);
					
					if(is_numeric($_POST['gcm_project_key']))
					{
						
						if($verify_gcm_result['verified'])
						{
							//Update GCM/FCM 
							$data = array(
								'gcm_project_key' => sanitize_text_field($_POST['gcm_project_key']),
								'gcm_api_key' => sanitize_text_field($_POST['gcm_api_key'])
							);

							$result = Api_class::updateFCMSettings($app_key, $data);

							$verify_gcm_result['banner'] = 'FCM settings updated successfully';
							$gcmdata = array(
								'gcm_project_key' => sanitize_text_field($_POST['gcm_project_key']),
								'gcm_api_key'	=> sanitize_text_field($_POST['gcm_api_key'])
							);
							$wp_session['gcmdata'] = (object)$gcmdata;
						}
						else
						{
							echo $error_message = $verify_gcm_data['banner'];
						}
					}
					else
					{
						echo $error_message = "FCM Sender ID should be a number.";
					}
				}

				$menu_active_key=true;
			}

			if(!empty($wp_session) && isset($wp_session['check_auth']['success'])) {
				$menu_active_key =false;
			}

			require_once( PUSHENGAGE_PLUGIN_DIR . '/views/admin.php' );
		}

		public static function getSiteData($app_key)
		{
			if(empty($wp_session['sitedata']))
			{
				$sitedata = Api_class::getSiteinfo($app_key);
			}
			else
			$sitedata = $wp_session['sitedata'];

			return $sitedata;
		}

		public static function update_settings($data)
		{
			if(!empty($data))
			{
				$pushengage_settings = self::pushengage_settings();
				foreach($data as $key => $value){
					$pushengage_settings[$key] = $value;
				}
				return update_option('pushengage_settings', $pushengage_settings);
			}
			return true;
		}

		public static function pushengageDomainScripts()
		{
			$pushengage_settings = self::pushengage_settings();
			if(empty($pushengage_settings)) {
				return;
			}

			$app_key = $pushengage_settings['appKey'];
			$site_key = $pushengage_settings['site_key'];

			if(!empty($app_key) && !empty($site_key))
			{
				$pe_dynemic_js = 'https://clientcdn.pushengage.com/core/'.$site_key.'.js';
				wp_enqueue_script('core', $pe_dynemic_js, false, false, true);
			}
		}
		
		function pe_corescript_tag( $tag, $handle, $src ) {
			if ( $handle !== 'core' ) {
				return $tag;
			}

			return "<script src='$src' async></script>";
		}


		 public static function save_post_meta_data( $post_id )
		 {

			if (! current_user_can( 'edit_posts' ) )
			{
				return false;
			}
			else
			{
				$no_note = get_post_meta( $post_id, '_pe_override', true );
				if ( isset( $_POST['pushengage-override'] ) && ! $no_note )
				{
					$override_setting = sanitize_text_field( $_POST['pushengage-override'] );
					add_post_meta( $post_id, '_pe_override', $override_setting, true );
				}
				elseif ( ! isset( $_POST['pushengage-override'] ) && $no_note )
				{
					delete_post_meta( $post_id, '_pe_override' );
				}
				if ( isset( $_POST['pushengage-custom-msg'] ) ) {
					update_post_meta( $post_id, '_pushengage_custom_text', sanitize_text_field( $_POST['pushengage-custom-msg'] ) );
				}
				if(isset($_POST['pushengage-override']))
				{
					if(!empty( $_POST['pushengage-categories'] ))
					{
						$draft_segments_filter = array_map(sanitize_text_field, $_POST['pushengage-categories']);
						$draft_segments = implode(" ",$draft_segments_filter);
						$prev_segments = get_post_meta( $post_id, '_pe_draft_segments', true );
						update_post_meta( $post_id, '_pe_draft_segments', $draft_segments, $prev_segments );
					}
					else
					{
						delete_post_meta($post_id, '_pe_draft_segments');
					}


					$str = "";
					if ( isset($_POST['pushengage-categories']) && !empty( $_POST['pushengage-categories'] ) && (get_post_status( $post_id ) == 'future' || get_post_status( $post_id ) == 'inherit') )
					{
						$str_filter = array_map(sanitize_text_field, $_POST['pushengage-categories']);
						$str = implode(" ", $str_filter);
						add_post_meta( $post_id, '_sedule_notification', $str, true );
					}
					add_post_meta( $post_id, 'pe_override_scheduled', 1, true );
				}
				else
				{
					delete_post_meta($post_id, '_pe_draft_segments');
					delete_post_meta($post_id, '_sedule_notification');
					delete_post_meta($post_id, 'pe_override_scheduled');
				}
			}
		}

		public static function sendPostNotifications( $new_status, $old_status, $post )
		{

			if ( empty( $post ) || 'publish' !== $new_status || false === self::pushengage_active() ) {
				return;
			}
			if ( ! current_user_can( 'publish_posts' ) && ! DOING_CRON  ) {
				return;
			}
			// prevent multiple send within 30 seconds.
			if(get_post_meta($post->ID, 'pe_timestamp',true) >= date("Y-m-d H:i:s")) {
				return;
			}

			$pushengage_settings = self::pushengage_settings();
			$app_key = $pushengage_settings['appKey'];
			$all_post_types = $pushengage_settings['all_post_types'];

			$appdata = !empty($wp_session['appdata'])?$wp_session['appdata']:'';
			$site_name = !empty($appdata['site_name'])?$appdata['site_name']:$pushengage_settings['site_name'];
			$post_id = $post->ID;
			$post_type = get_post_type( $post );

			if ( false === $all_post_types )
			{
				if ( 'post' !== $post_type ) {
					return;
				}
			}
			if ( 'publish' === $new_status  )
			{
				if ( isset( $_POST['pushengage-override'] ) ) {
					$send_note = true;
				}
			}
			if('publish' === $new_status && 'future' === $old_status)
			{
				if(get_post_meta( $post_id, 'pe_override_scheduled', true ))
					$send_note = true;
			}
			if ( $new_status !== $old_status || ! empty( $send_note ) )
			{
				if ( 'publish' === $new_status )
				{
					$categories = get_the_category( $post_id );
					$auto_push = $pushengage_settings['autoPush'];
					$non_pe_categories = $pushengage_settings['categories'];
					$segment_send = $pushengage_settings['segment_send'];
					$use_featured_image = $pushengage_settings['use_featured_image'];
					$segments = null;
					$image_url = null;

					if ( ( 'publish' === $new_status && 'future' === $old_status ))
					{
						$override = get_post_meta( $post_id, '_pe_override', true );
						$custom_headline = get_post_meta( $post_id, '_pushengage_custom_text', true );
						$segments=explode(" ",get_post_meta( $post_id, '_sedule_notification', true ));
						$seg_array = array_filter($segments);
						if(empty($seg_array))
						$segments=false;
					}
					else
					{
						if ( isset( $_POST['pushengage-override'] ) ) {
							$override = sanitize_text_field( $_POST['pushengage-override'] );
						}
						if ( isset( $_POST['pushengage-custom-msg'] ) && ! empty( $_POST['pushengage-custom-msg'] ) ) {
							$custom_headline = sanitize_text_field( $_POST['pushengage-custom-msg'] );
						}
					}
					if ( $send_note )
					{
						$adv_options = array();
						if ( !empty( $override ) )
						{
							if ( isset($_POST['pushengage-categories']) && !empty( $_POST['pushengage-categories'] ) )
							{
								$segments = array_map( sanitize_text_field, $_POST['pushengage-categories']);
							}
							if ( ! empty( $custom_headline ) ) {
								$note_text = stripslashes( $custom_headline ) ;
							} else {
								$note_text = sanitize_text_field(substr(get_the_title( $post_id ),0,72));
							}
							$note_link = get_permalink( $post_id );
							if ( has_post_thumbnail( $post_id ) ) {
								$raw_image = wp_get_attachment_image_src( get_post_thumbnail_id( $post_id ) );
								$featured_image_url = $raw_image[0];
							}
							if ( !empty($use_featured_image) ) {
								$image_url = !empty($featured_image_url)?$featured_image_url:'';
							}
							else
							{
								$image_url = !empty($appdata['site_image'])?$appdata['site_image']:'';
							}
							if(empty($use_featured_image)){
								$adv_options['big_image_url'] = $featured_image_url;
							}

							//add UTM params for link
							if(isset($pushengage_settings['utmcheckbox']) 
								&& !empty($pushengage_settings['utmcheckbox'])
								&& isset($pushengage_settings['utm_source']) 
								&& isset($pushengage_settings['utm_medium']) 
								&& isset($pushengage_settings['utm_campaign'])
							)
							{
								$note_link = add_query_arg( array(
    								'utm_source' => $pushengage_settings['utm_source'],
    								'utm_medium' => $pushengage_settings['utm_medium'],
    								'utm_campaign' => $pushengage_settings['utm_campaign']
								), $note_link );
							}
							else
							{
								$note_link = add_query_arg( array(
    								'utm_source' => 'pushengage',
    								'utm_medium' => 'push_notification',
    								'utm_campaign' => 'pushengage'
								), $note_link);	
							}
							

							if(empty($appdata['site_name']))
							{
								$appdata['site_name'] = $pushengage_settings['site_name'];
							}

							//query params post_id, old status and new status is for debugging purpose.
							$adv_options['post_id'] = $post_id;
							$adv_options['old_status'] = $old_status;
							$adv_options['new_status'] = $new_status;
							
							//add or update post metadata "pe_timestamp"  by 30 second, whenever notification sent.
							if(get_post_meta($post->ID, 'pe_timestamp',true)) {
								update_post_meta($post->ID, 'pe_timestamp', date("Y-m-d H:i:s", time() + 30));
							} else {
								add_post_meta($post->ID, 'pe_timestamp', date("Y-m-d H:i:s", time() + 30), true);
							}

							$result = Api_class::send_notification( $note_text, $note_link, $app_key, $site_name, $segments, $image_url, $adv_options );
						}
					}
				}
			}
		}

		public static function filter_string( $string )
		{
			$string = str_replace( '&#8220;', '&quot;', $string );
			$string = str_replace( '&#8221;', '&quot;', $string );
			$string = str_replace( '&#8216;', '&#39;', $string );
			$string = str_replace( '&#8217;', '&#39;', $string );
			$string = str_replace( '&#8211;', '-', $string );
			$string = str_replace( '&#8212;', '-', $string );
			$string = str_replace( '&#8242;', '&#39;', $string );
			$string = str_replace( '&#8230;', '...', $string );
			$string = str_replace( '&prime;', '&#39;', $string );

			return html_entity_decode( $string, ENT_QUOTES );
		}

		public static function chekAPIKey()
		{
			$pushengage_settings = self::pushengage_settings();
			$app_key = $pushengage_settings['appKey'];
			if($app_key) 
			{
				return true;
			}
			else{
				return false;
			} 
		}

		public function register_session()
		{
			if(!session_id())
			{
				session_start();
			}
		}

		public static function checkWPVersion()
		{
			$user_version = self::$pushengage_version;

			//Get current wordpres information
			$plugin_name = 'pushengage';
			$args = array( 
				'timeout' => 15,
				'redirection' => 30, 
			);
		    $url = 'http://api.wordpress.org/plugins/info/1.0/' . $plugin_name . '.json';
		    $response = wp_remote_get( $url, $args );
		    if(is_array($response) && !empty($response['body'])) 
		    {
		    	$plugin_info = json_decode( $response['body'] );
		    }
		    $current_version = !empty($plugin_info->version) ? $plugin_info->version : '';
		    if(!empty($user_version) && !empty($current_version) && $user_version != $current_version)
		    {
		    	return false;
		    }
		    return true;
		}

		//get ISO timezone, if user has saved the timezone in UP and UM form.
		public static function getIsoTimezone($timezone)
		{
		    $timezone_mapping=[
		        "UM12" => "UTC",
		        "UM11" => "Pacific/Niue",
		        "UM10" => "Pacific/Rarotonga",
		        "UM95" => "Pacific/Marquesas",
		        "UM9" => "Pacific/Gambier",
		        "UM8" => "Pacific/Pitcairn",
		        "UM7" => "America/Creston",
		        "UM6" => "America/Belize",
		        "UM5" => "America/Chicago",
		        "UM45"=>"America/Caracas",
		        "UM4"=>"America/St_Vincent",
		        "UM35"=>"America/St_Johns",
		        "UM3"=>"America/Argentina/La_Rioja",
		        "UM2"=>"America/Noronha",
		        "UM1"=>"Atlantic/Cape_Verde",
		        "UTC"=>"UTC",
		        "UP1"=>"Africa/Algiers",
		        "UP2"=>"Europe/Amsterdam",
		        "UP3"=>"Africa/Addis_Ababa",
		        "UP35"=>"Asia/Tehran",
		        "UP4"=>"Europe/Samara",
		        "UP45"=>"Asia/Kabul",
		        "UP5"=>"Asia/Karachi",
		        "UP55"=>"Asia/Kolkata",
		        "UP575"=>"Asia/Kathmandu",
		        "UP6"=>"Asia/Dhaka",
		        "UP65"=>"Asia/Yangon",
		        "UP7"=>"Asia/Vientiane",
		        "UP8"=>"Asia/Macau",
		        "UP875"=>"Australia/Eucla",
		        "UP9"=>"Asia/Tokyo",
		        "UP95"=>"Australia/Broken_Hill",
		        "UP10"=>"Australia/Currie",
		        "UP105"=>"Australia/Lord_Howe",
		        "UP11"=>"Asia/Srednekolymsk",
		        "UP115"=>"Pacific/Norfolk",
		        "UP12"=>"Pacific/Fiji",
		        "UP1275"=>"Pacific/Chatham",
		        "UP13"=>"Pacific/Tongatapu",
		        "UP14"=>"Pacific/Kiritimati",			
		    ];
		
		    if (isset($timezone_mapping[$timezone])) {
		        return $timezone_mapping[$timezone];
		    }
		
		    try {
		        $iso_timezone = new DateTimeZone($timezone);
		        return $timezone;
		    } catch (Exception $e) {
		        return 'UTC';
		    }
		}

		//dropdown list of timezone in ISO form along with its offset values	
		public static function select_Timezone($selected = '') 
		{
			$iso_timezone_list = timezone_identifiers_list();
			$timezone_offsets = array();
			foreach($iso_timezone_list as $timezone){
				$tz= new DateTime('now', new DateTimeZone($timezone));
				$timezone_offsets[$timezone]=$tz->format('P');
			}
			asort($timezone_offsets);
			$select= '<select name="timezones" class="form-control">';
			foreach($timezone_offsets as $iso_timezone => $offset){	
				$select .= '<option value="'.$iso_timezone.'"';
				$select .= ($iso_timezone == $selected ? 'selected' : '');
				$select .= '>'."(GMT".$offset."  ".$iso_timezone.")".'</option>';	
			}
			$select .= '</select>';
			return $select;
		}

		public static function init_user_options() 
		{
			if ( !is_user_logged_in() ) 
			{
				return;
			}

			// options for logged in user
			add_action( 'add_meta_boxes', array( __CLASS__, 'note_override_add_meta_box' ) );
			add_action( 'add_meta_boxes', array( __CLASS__, 'custom_note_text' ), 10, 2 );
			add_action( 'save_post', array( __CLASS__, 'save_post_meta_data' ) );

			if ( self::can_edit_plugin_options() ) 
			{
    			add_action( 'admin_init', array( __CLASS__, 'validate_csrf_token') );
    			add_action( 'admin_init', array( __CLASS__, 'pushengage_save_settings' ) );
				add_action( 'admin_menu', array( __CLASS__, 'admin_menu_add' ) );
			}
		}

		public static function validate_csrf_token() {
			if ( !empty($_POST)
				&& $_GET['page'] == 'pushengage-admin'
				&& (
					! isset( $_POST['pe_token'] )
					|| ! wp_verify_nonce($_POST['pe_token'], 'pe_token')
					|| ! self::can_edit_plugin_options()
				)
			)
			{
				$tab = !empty($_GET['tab']) ? sanitize_text_field($_GET['tab']) : 'gSettings';
				$url = esc_url_raw( admin_url( 'admin.php?page=pushengage-admin&tab=' . $tab . '&status=failed'));
				wp_redirect($url);
				exit;
			}
		}

		private static function can_edit_plugin_options() 
		{
			return current_user_can('manage_options');
		}
	}
?>
